use chrono::Utc;
use gtk::prelude::*;
use log::debug;
use relm4::{
    component::{AsyncComponent, AsyncComponentParts},
    prelude::*,
    AsyncComponentSender,
};

use crate::{
    database::Database,
    sheet::{I64DateTime, Sheet},
    sheet_dao,
    ui::mcdu::McduOutput,
};

use super::{
    mcdu::McduModel,
    sheet_listing::{SheetListingInput, SheetListingModel},
};

pub struct AppModel {
    database: Database,
    mcdu: Controller<McduModel>,
    sheets_listing: Controller<SheetListingModel>,
    edit_mode: bool,
}

#[derive(Debug)]
pub enum AppInput {
    SearchStarted(String),
    SheetPressed(Sheet),
    Refresh,
    Sort,
    Shuffle,
    SetEditMode(bool),
}

pub struct AppInitData {
    pub sheets: Vec<Sheet>,
    pub database: Database,
}

#[relm4::component(pub, async)]
impl AsyncComponent for AppModel {
    type Input = AppInput;
    type Output = ();
    type Init = AppInitData;
    type CommandOutput = ();

    view! {
        #[root]
        gtk::Window{
            set_default_width: 300,
            set_default_height: 300,
            set_title: Some("Play music!!!"),
            set_maximized: true,
            gtk::Box {
                set_orientation: gtk::Orientation::Horizontal,
                gtk::Box {
                    set_orientation: gtk::Orientation::Vertical,
                    set_hexpand: true,
                    gtk::Box {
                        set_orientation: gtk::Orientation::Horizontal,
                        set_margin_all: 10,
                        set_spacing: 3,
                        gtk::Button {
                            set_icon_name: "view-refresh-symbolic",
                            set_margin_end: 10,
                            connect_clicked[sender] => move |_| sender.input(AppInput::Refresh),
                        },
                        gtk::ToggleButton {
                            set_icon_name: "document-edit-symbolic",
                            set_margin_end: 10,
                            connect_clicked[sender] => move |button| sender.input(AppInput::SetEditMode(button.is_active())),
                        },
                        #[name = "button_sort"]
                        gtk::ToggleButton {
                            set_icon_name: "view-sort-descending-symbolic",
                            set_active: true,
                            connect_clicked[sender] => move |_| sender.input(AppInput::Sort),
                        },
                        gtk::ToggleButton {
                            set_icon_name: "media-playlist-shuffle-symbolic",
                            set_group: Some(&button_sort),
                            connect_clicked[sender] => move |_| sender.input(AppInput::Shuffle),
                        },
                    },
                    gtk::ScrolledWindow {
                            model.sheets_listing.widget(),
                            set_vexpand: true,
                            set_hexpand: true,
                    },
                },
                model.mcdu.widget() {
                    set_margin_all: 10,
                },
            }
        }
    }

    async fn init(
        init_data: Self::Init,
        window: Self::Root,
        sender: AsyncComponentSender<Self>,
    ) -> AsyncComponentParts<Self> {
        relm4_icons::initialize_icons();

        let mcdu = McduModel::builder()
            .launch(())
            .forward(sender.input_sender(), |response| match response {
                McduOutput::SearchStarted(query) => AppInput::SearchStarted(query),
            });

        let mut sheets = init_data.sheets;
        sheets.sort_by(|a, b| a.cmp(b).reverse());

        let sheets_listing = SheetListingModel::builder()
            .launch(sheets)
            .forward(sender.input_sender(), |response| {
                AppInput::SheetPressed(response.sheet)
            });

        let model = AppModel {
            database: init_data.database,
            mcdu,
            sheets_listing,
            edit_mode: false,
        };

        let widgets = view_output!();

        AsyncComponentParts { model, widgets }
    }

    async fn update(
        &mut self,
        message: Self::Input,
        _sender: AsyncComponentSender<Self>,
        _root: &Self::Root,
    ) {
        match message {
            AppInput::SearchStarted(query) => {
                self.sheets_listing
                    .emit(SheetListingInput::Query(query.clone()));
            }
            AppInput::SheetPressed(sheet) => {
                if self.edit_mode {
                    debug!("Sheet pressed, but we are in edit mode!");
                } else {
                    sheet.open_file();
                    let mut sheet = sheet;
                    sheet.last_opened = I64DateTime(Utc::now());
                    sheet_dao::update_sheet_last_opened(&self.database, &sheet)
                        .await
                        .unwrap();
                }
            }
            AppInput::Refresh => todo!(),
            AppInput::Sort => self.sheets_listing.emit(SheetListingInput::Sort),
            AppInput::Shuffle => self.sheets_listing.emit(SheetListingInput::Shuffle),
            AppInput::SetEditMode(edit_mode) => self.edit_mode = edit_mode,
        }
    }
}
